import React, { useMemo, useState, useRef, forwardRef, useImperativeHandle } from 'react'
import { Table, Tooltip, Button, Popconfirm, Input, Space, Tag } from 'antd'
import { TweenOneGroup } from 'rc-tween-one'
import styles from './index.less'
import { SearchOutlined } from '@ant-design/icons'
import ResizeableTitle from './components/ResizeableTitle'
import FileView from './components/FileView'
import CoordView from './components/CoordView'
import SelectView from './components/SelectView'
import IconPack from '../../../components/IconPack'
import { isObject } from '../../../../utils'

const getFileInfo = (formJson) => {
  let obj = {}
  let parent = formJson?.properties
  if (isObject(parent)) {
    for (let v in parent) {
      let child = parent[v]?.properties
      if (isObject(child)) {
        for (let s in child) {
          obj[s] = child[s]
        }
      }
    }
  }
  return obj
}

const TablePack = (props, ref) => {

  useImperativeHandle(ref, () => ({
    setFilteredInfo,
    setSortedInfo,
  }))

  const { parent, readOnly, config, fieldList, loading, dataSource, rowSelection, onRow, tableChange, btnsClick } = props
  const [fieldResize, setFieldResize] = useState({})
  const [filteredInfo, setFilteredInfo] = useState({})
  const [sortedInfo, setSortedInfo] = useState({})

  const filtered = useMemo(() => {
    let array = []
    if (isObject(filteredInfo)) {
      Object.keys(filteredInfo).forEach(v => {
        if (Array.isArray(filteredInfo[v])) {
          array.push({ filterName: v, filterValue: filteredInfo[v].join(',') })
        }
      })
    }
    return array
  }, [filteredInfo])

  //台账列表、台账选择器表头
  const getColumnProps = (json, v) => {
    let searchProps = {
      filterDropdown: ({ setSelectedKeys, selectedKeys, confirm, clearFilters, close }) => {
        return (
          <div
            style={{ padding: 8 }}
            onKeyDown={(e) => e.stopPropagation()}
          >
            <Input
              placeholder={`搜索 ${v}`}
              value={selectedKeys[0]}
              onChange={e => {
                setSelectedKeys(e.target.value ? [e.target.value] : [])
              }}
              onPressEnter={() => {
                confirm({ closeDropdown: true })
              }}
              style={{
                marginBottom: 8,
                display: 'block',
                width: '280px',
              }}
            />
            <Space>
              <Button
                type="primary"
                onClick={() => {
                  confirm({ closeDropdown: true })
                }}
                icon={<SearchOutlined />}
                size="small"
                style={{ width: 90 }}
              >
                搜索
              </Button>
              <Button
                onClick={() => {
                  clearFilters()
                  confirm({ closeDropdown: true })
                }}
                size="small"
                style={{ width: 90 }}
              >
                重置
              </Button>
            </Space>
          </div>
        )
      },
      filterIcon: (filtered) => (
        <SearchOutlined
          style={{
            color: filtered ? '#1890ff' : undefined,
          }}
        />
      ),
    }
    const { widget, sourceType, options } = json?.[v]
    if (['CheckBox', 'ComboBox', 'RadioButton'].includes(widget)) {
      if (sourceType === '手动输入') {
        searchProps = {
          filters: options.map(v => { return { text: v.value, value: v.value, } }),
        }
      }
    }

    return {
      title: json[v]?.title || v,
      dataIndex: v,
      width: fieldResize[v] || 120,
      ellipsis: true,
      sorter: true,
      sortOrder: sortedInfo.field === v ? sortedInfo.order : null,
      filteredValue: filteredInfo[v] || null,
      ...searchProps,
      onHeaderCell: (column) => ({
        width: column.width,
        onResize: (e, props) => {
          setFieldResize({ ...fieldResize, [v]: props?.size?.width })
        },
      }),
      render: (value, r) => {
        let props = { ...json[v], value }
        if (widget === 'FileUpload') {
          return <FileView {...props} />
        }
        if (['Coordinate', 'Device', 'DrawPath', 'DrawArea'].includes(widget)) {
          return <CoordView {...props} />
        }
        if (['RadioButton', 'CheckBox'].includes(widget)) {
          return <SelectView {...props} />
        }
        if (value && value.length > 10) {
          return (
            <Tooltip title={value}>
              <span style={{ display: 'inline-block', width: '100%' }}>{value}</span>
            </Tooltip>
          )
        }
        return (
          <div>{value}</div>
        )
      },
    }
  }

  //关联表单表头
  const getRelevanceColumnProps = (json, v) => {
    const { widget } = json?.[v] || {}
    return {
      title: json[v]?.title || v,
      dataIndex: v,
      width: fieldResize[v] || 120,
      ellipsis: true,
      sorter: (a, b) => a[v] - b[v],
      filteredValue: filteredInfo[v] || null,
      filterDropdown: ({ setSelectedKeys, selectedKeys, confirm, clearFilters, close }) => (
        <div
          style={{ padding: 8 }}
          onKeyDown={(e) => e.stopPropagation()}
        >
          <Input
            ref={searchInput}
            placeholder={`Search ${v}`}
            value={selectedKeys[0]}
            onChange={(e) => setSelectedKeys(e.target.value ? [e.target.value] : [])}
            onPressEnter={() => {
              confirm();
            }}
            style={{
              marginBottom: 8,
              display: 'block',
            }}
          />
          <Space>
            <Button
              type="primary"
              onClick={() => {
                confirm();
              }}
              icon={<SearchOutlined />}
              size="small"
              style={{
                width: 90,
              }}
            >
              搜索
            </Button>
            <Button
              onClick={() => {
                clearFilters();
                confirm();
              }}
              size="small"
              style={{
                width: 90,
              }}
            >
              重置
            </Button>
          </Space>
        </div>
      ),
      filterIcon: (filtered) => (
        <SearchOutlined
          style={{
            color: filtered ? '#1890ff' : undefined,
          }}
        />
      ),
      onFilter: (value, record) =>
        record[v].toString().toLowerCase().includes(value.toLowerCase()),
      onFilterDropdownOpenChange: (visible) => {
        if (visible) {
          setTimeout(() => searchInput.current?.select(), 100);
        }
      },
      onHeaderCell: (column) => ({
        width: column.width,
        onResize: (e, props) => {
          setFieldResize({ ...fieldResize, [v]: props?.size?.width })
        },
      }),
      render: (value, r) => {
        let props = { ...json[v], value }
        if (widget === 'FileUpload') {
          return <FileView {...props} />
        }
        if (['Coordinate', 'Device', 'DrawPath', 'DrawArea'].includes(widget)) {
          return <CoordView {...props} />
        }
        if (['RadioButton', 'CheckBox'].includes(widget)) {
          return <SelectView {...props} />
        }
        if (value && value.length > 10) {
          return (
            <Tooltip title={value}>
              <span style={{ display: 'inline-block', width: '100%' }}>{value}</span>
            </Tooltip>
          )
        }
        return (
          <div>{value}</div>
        )
      }
    }
  }

  const btns = useMemo(() => {
    if (readOnly) {
      return [
        {
          title: '详情',
          icon: <IconPack.XiangQing />,
        },
      ]
    }
    return [
      {
        title: '详情',
        icon: <IconPack.XiangQing />,
      },
      {
        title: '编辑',
        icon: <IconPack.BianJi />,
      },
      {
        title: '删除',
        icon: <IconPack.ShanChu />,
      }
    ]
  }, [readOnly, parent])

  const fileColumns = useMemo(() => {
    const { webShowFieldGroup, formJson } = config
    let showField = []
    if (Array.isArray(fieldList) && fieldList.length > 0) {
      showField = fieldList
    } else {
      showField = webShowFieldGroup ? webShowFieldGroup.split(',').filter(v => v) : []
    }
    let json = getFileInfo(formJson)
    let array = []
    showField.forEach((v, i) => {
      if (parent === '关联表单') {
        array.push(getRelevanceColumnProps(json, v))
      } else {
        array.push(getColumnProps(json, v))
      }
    })
    return array
  }, [parent, config, fieldList, fieldResize, filteredInfo, sortedInfo])

  const columns = useMemo(() => {
    let columns = [
      {
        title: '序号',
        width: 50,
        align: 'center',
        filteredValue: null,
        render: (_, r, i) => i + 1
      },
      ...fileColumns,
      {
        title: '操作',
        width: btns.length * 35 + 30,
        align: 'center',
        filteredValue: null,
        fixed: 'right',
        render: (r) => {
          return (
            <div style={{ display: 'flex', justifyContent: 'center' }}>
              {
                btns.map(v => {
                  if (v.title === '删除') {
                    return (
                      <Popconfirm
                        key={v.title}
                        title='确定要删除数据吗？'
                        onConfirm={() => btnsClick({ type: v.title, row: r })}
                        okText='确定'
                        cancelText='取消'
                      >
                        <Button
                          type='link'
                          title='删除'
                          style={{ padding: '0 7px' }}
                          icon={v.icon}
                        />
                      </Popconfirm>
                    )
                  }
                  return (
                    <Button
                      key={v.title}
                      type='link'
                      title={v.title}
                      icon={v.icon}
                      style={{ padding: '0 7px' }}
                      onClick={() => btnsClick({ type: v.title, row: r })}
                    >
                    </Button>
                  )
                })
              }
            </div>
          )
        }
      }
    ]
    if (parent === '台账选择器') {
      return fileColumns
    }
    return columns
  }, [parent, btns, fileColumns])

  const onChange = (page, filters, sorter) => {
    setFilteredInfo(filters)
    setSortedInfo(sorter)
    tableChange?.(page, filters, sorter)
  }

  const closeTag = (e, { filterName, filterValue }) => {
    e.preventDefault()
    let filtered = {}
    for (let key in filteredInfo) {
      if (key === filterName) {
        filtered[key] = null
      } else {
        filtered[key] = filteredInfo[key]
      }
    }
    setFilteredInfo(filtered)
    tableChange?.({}, filtered, sortedInfo)
  }

  return (
    <div className={styles.tablePack} style={{ paddingTop: filtered.length ? '30px' : '10px' }}>
      {
        filtered.length ? (
          <div className={styles.condition}>
            <TweenOneGroup
              enter={{
                scale: 0.8,
                opacity: 0,
                type: 'from',
                duration: 200,
              }}
              onEnd={(e) => {
                if (e.type === 'appear' || e.type === 'enter') {
                  e.target.style = 'display: inline-block';
                }
              }}
              leave={{
                opacity: 0,
                width: 0,
                scale: 0,
                duration: 200,
              }}
              appear={false}
            >
              {
                filtered.map(v => {
                  return (
                    <span
                      key={v.filterName}
                      style={{ display: 'inline-block' }}
                    >
                      <Tag
                        color='volcano'
                        closable
                        onClose={(e) => closeTag(e, v)}
                      >
                        <span>{v.filterName}</span>
                        <span style={{ padding: '0 3px' }}>:</span>
                        <span>{v.filterValue}</span>
                      </Tag>
                    </span>
                  )
                })
              }
            </TweenOneGroup>
          </div>
        ) : null
      }
      <Table
        size='small'
        rowKey='ID'
        bordered
        loading={loading}
        columns={columns}
        dataSource={dataSource}
        rowSelection={rowSelection}
        onRow={onRow}
        pagination={parent === '关联表单' ? { total: dataSource.length, showTotal: (value) => `总计 ${value} 条` } : false}
        scroll={{ x: '100%', y: 'calc(100% - 40px)' }}
        onChange={onChange}
        components={{
          header: {
            cell: ResizeableTitle
          }
        }}
      />
    </div>
  )

}

export default forwardRef(TablePack)